﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2018 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.DataAnalysis {
  [Item("Time Series Prognosis Transformation Model", "A time series prognosis model that was transformed back to match the original variables after the training was performed on transformed variables.")]
  [StorableClass]
  public class TimeSeriesPrognosisTransformationModel : RegressionTransformationModel, ITimeSeriesPrognosisTransformationModel {

    public new ITimeSeriesPrognosisModel OriginalModel {
      get { return (ITimeSeriesPrognosisModel)base.OriginalModel; }
    }

    #region Constructor, Cloning & Persistence
    public TimeSeriesPrognosisTransformationModel(ITimeSeriesPrognosisModel originalModel, IEnumerable<IDataAnalysisTransformation> transformations)
      : base(originalModel, transformations) {
    }

    protected TimeSeriesPrognosisTransformationModel(TimeSeriesPrognosisTransformationModel original, Cloner cloner)
      : base(original, cloner) {
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new TimeSeriesPrognosisTransformationModel(this, cloner);
    }

    [StorableConstructor]
    protected TimeSeriesPrognosisTransformationModel(bool deserializing)
      : base(deserializing) { }
    #endregion


    public IEnumerable<IEnumerable<double>> GetPrognosedValues(IDataset dataset, IEnumerable<int> rows, IEnumerable<int> horizons) {
      var transformedInput = DataAnalysisTransformation.Transform(dataset, InputTransformations);
      var estimates = OriginalModel.GetPrognosedValues(transformedInput, rows, horizons);
      return estimates.Select(x => InverseTransform(x, TargetTransformations));
    }

    public ITimeSeriesPrognosisSolution CreateTimeSeriesPrognosisSolution(ITimeSeriesPrognosisProblemData problemData) {
      return new TimeSeriesPrognosisSolution(this, problemData);
    }
  }
}